/***************************************************************************
*
*     Project:      Firefly X10 Board
*
*     File:         x10io.h
*
*     Version:      1.3
*
*     Description:  Definitions relating specifically to IO.
*
*     Changes:
*     1.3 MJB       Defined new structure ReelStatusEx which is used by
*                   API function GetReelStatusEx(). This replaces the
*                   old usbReelStatus structure. The new structure uses
*                   a WORD instead of a BYTE for reel position and
*                   error, 10/2/2004
*     1.2 MJB       Defined MAX_RAMP_TABLE_LENGTH. This is the maximum
*                   ramp table length as used in structure usbRampTable.
*                   Was 10 now 60, 26/1/2004
*     1.1 MJB       Reformatted and added Parallel I/O stuff, 14/3/2003
*     1.0 MJB       First version, 11/03/2003
*
***************************************************************************/

/********    Header Sentry    ********/
#ifndef X10IO_H
#define X10IO_H

////////////////////////////////////////////////////////////////////////////
// Inputs                                                                 //
////////////////////////////////////////////////////////////////////////////

#define ON 1
#define OFF 0

#define USB_IN_0		0
#define USB_IN_1		1
#define USB_IN_2		2
#define USB_SW			3
#define USB_CS			4
#define USB_IN_LENGTH   5

#define USB_MUXIN_CHANNELS (4)
#define USB_MUXIN_BYTES (3)

typedef enum _usbInputBitId
{
    INPUT_BIT_IP0, INPUT_BIT_IP1, INPUT_BIT_IP2, INPUT_BIT_IP3,
    INPUT_BIT_IP4, INPUT_BIT_IP5, INPUT_BIT_IP6, INPUT_BIT_IP7,
    INPUT_BIT_IP8, INPUT_BIT_IP9, INPUT_BIT_IP10, INPUT_BIT_IP11,
    INPUT_BIT_IP12, INPUT_BIT_IP13, INPUT_BIT_IP14, INPUT_BIT_IP15,
    INPUT_BIT_IP16, INPUT_BIT_IP17, INPUT_BIT_IP18, INPUT_BIT_IP19,
    INPUT_BIT_IP20, INPUT_BIT_IP21, INPUT_BIT_IP22, INPUT_BIT_IP23,
    INPUT_BIT_SW0, INPUT_BIT_SW1, INPUT_BIT_SW2, INPUT_BIT_SW3,
    INPUT_BIT_SW4, INPUT_BIT_SW5, INPUT_BIT_SW6, INPUT_BIT_SW7,
    INPUT_BIT_CURRENT_SENSE
} usbInputBitId;

typedef struct
{
    BYTE	byIn[3];		// Inputs [0..2]
    BYTE	bySw;			// Sw input
    BYTE	byCs;			// Cs input
} usbInput;


typedef struct
{
    BYTE byMuxStatus;
    BYTE byMuxInp[ USB_MUXIN_CHANNELS ][ USB_MUXIN_BYTES ];
} usbMultiplexedInput;

typedef enum
{
    InputMultiplexDisabled,
    InputMultiplexEnabled
} usbInputMultiplexing;


////////////////////////////////////////////////////////////////////////////
// Outputs                                                                //
////////////////////////////////////////////////////////////////////////////

#define USB_OUT_0		0
#define USB_OUT_1		1
#define USB_OUT_2		2
#define USB_OUT_3		3
#define USB_AUX			4
#define USB_OUT_LENGTH  	5

typedef enum _usbOutputBitId
{
    OUTPUT_BIT_OP0, OUTPUT_BIT_OP1, OUTPUT_BIT_OP2, OUTPUT_BIT_OP3,
    OUTPUT_BIT_OP4, OUTPUT_BIT_OP5, OUTPUT_BIT_OP6, OUTPUT_BIT_OP7,
    OUTPUT_BIT_OP8, OUTPUT_BIT_OP9, OUTPUT_BIT_OP10, OUTPUT_BIT_OP11,
    OUTPUT_BIT_OP12, OUTPUT_BIT_OP13, OUTPUT_BIT_OP14, OUTPUT_BIT_OP15,
    OUTPUT_BIT_OP16, OUTPUT_BIT_OP17, OUTPUT_BIT_OP18, OUTPUT_BIT_OP19,
    OUTPUT_BIT_OP20, OUTPUT_BIT_OP21, OUTPUT_BIT_OP22, OUTPUT_BIT_OP23,
    OUTPUT_BIT_OP24, OUTPUT_BIT_OP25, OUTPUT_BIT_OP26, OUTPUT_BIT_OP27,
    OUTPUT_BIT_OP28, OUTPUT_BIT_OP29, OUTPUT_BIT_OP30, OUTPUT_BIT_OP31,
    OUTPUT_BIT_AUX0, OUTPUT_BIT_AUX1, OUTPUT_BIT_AUX2, OUTPUT_BIT_AUX3,
    OUTPUT_BIT_AUX4, OUTPUT_BIT_AUX5
} usbOutputBitId;

typedef enum _usbOutputId
{
    USB_OP_0,
    USB_OP_1,
    USB_OP_2,
    USB_OP_3,
    USB_OP_AUX
} usbOutputId;

typedef struct
{
    BYTE	byOut[4];		// Outputs [0..3]
    BYTE	byAux;			// Aux output
} usbOutput;


////////////////////////////////////////////////////////////////////////////
// Reels                                                                  //
////////////////////////////////////////////////////////////////////////////

#define MAX_RAMP_TABLE_LENGTH		60

// Obsolete reel status structure used by GetReelStatus()
typedef struct
{
    BYTE	byPosition[3];
    BYTE	byError[3];
    BYTE	byBusy[3];
    BYTE	bySynchronised[3];
} usbReelStatus;

// New reel status structure used by GetReelStatusEx(). Use this if possible.
typedef struct
{
    WORD	position[3];
    WORD	error[3];
    BYTE	busy[3];
    BYTE	synchronised[3];
} ReelStatusEx;

typedef struct
{
    BYTE	num_entries;						// Number of entries
    BYTE	ramp_code[MAX_RAMP_TABLE_LENGTH];	// Ramp table codes
} usbRampTable;


////////////////////////////////////////////////////////////////////////////
// Parallel I/O                                                           //
////////////////////////////////////////////////////////////////////////////

typedef enum { Running, Success, Failure } ReleaseHopperCoinsStatus;
typedef enum { Low, High } ActiveState;

#endif

