/*****************************************************************
*
*     Project:      Firefly X10 Board
*
*     File:         x10serl.h
*
*     Version:      1.0
*
*     Description:  Definitions relating specifically to serial.
*
*     Changes:
*     1.0 MJB       First version, 28/2/2003
*
*****************************************************************/

/********    Header Sentry    ********/
#ifndef X10SERL_H
#define X10SERL_H

#define CBR_110                 110
#define CBR_300                 300
#define CBR_600                 600
#define CBR_1200                1200
#define CBR_2400                2400
#define CBR_4800                4800
#define CBR_9600                9600
#define CBR_14400               14400
#define CBR_19200               19200
#define CBR_38400               38400
#define CBR_57600               57600
#define CBR_62500				62500

//
// RTS Control Flow Values
//
#ifndef RTS_CONTROL_DISABLE
#define RTS_CONTROL_DISABLE		0x00
#endif
#ifndef RTS_CONTROL_ENABLE
#define RTS_CONTROL_ENABLE		0x01
#endif
#ifndef RTS_CONTROL_HANDSHAKE
#define RTS_CONTROL_HANDSHAKE	0x02
#endif
#ifndef RTS_CONTROL_TOGGLE
#define RTS_CONTROL_TOGGLE		0x03
#endif

#ifndef NOPARITY
#define NOPARITY				0
#endif
#ifndef ODDPARITY
#define ODDPARITY				1
#endif
#ifndef EVENPARITY
#define EVENPARITY				2
#endif
#ifndef MARKPARITY
#define MARKPARITY				3
#endif
#ifndef SPACEPARITY
#define SPACEPARITY				4
#endif

// Types
typedef enum _usbSerialPort { PORT_A, PORT_B } usbSerialPort;
typedef enum _usbPortType { PORT_RS232, PORT_RS232_POLLED, PORT_CCTALK, PORT_CCTALK_MODE1 } usbPortType;

// CCTalk polling method
typedef enum _PollMethod { Disabled, Once, Repeated, Triggered } PollMethod;

// CCTalk configuration
#define MAX_POLL_MSG_LENGTH	25	// Max. poll and inhibit message length for polled mode

#define NO_TRIGGER		4	// Use this in parameter 'next_trigger_device'
							// in structure 'RS232PollConfig' if you do
							// not need to trigger another device

typedef struct
{
    BYTE device_number;						// Device number (0-3)
    PollMethod method;						// Poll method: Disabled, Once or Repeated
    BYTE next_trigger_device;				// Next device to trigger. Set to NO_TRIGGER if not needed.
    BYTE poll_retry_count;					// Number of retries before sending an inhibit message
    int polling_interval;					// Polling interval (0-2550ms)
    WORD max_response_time;					// Max time to wait for response (0-65535ms)
    BYTE min_buffer_space;					// Min receive buffer space allowed
    BYTE poll_msg[MAX_POLL_MSG_LENGTH];		// Poll message
    BYTE inhibit_msg[MAX_POLL_MSG_LENGTH];	// Device inhibit message
} CCTalkConfig;

typedef struct
{
    BYTE device_number;						// Device number (0-3)
    PollMethod method;						// Poll method: Disabled, Once or Repeated
    BYTE next_trigger_device;				// Next device to trigger. Set to NO_TRIGGER if not needed.
    BYTE poll_retry_count;					// Number of retries before sending an inhibit message
    int polling_interval;					// Polling interval (0-2550ms)
    BYTE remove_local_echo;					// Remove local echo? TRUE or FALSE
    BYTE length_byte_offset;				// Offset to length byte in message
    BYTE add_to_length_byte;				// Offset to received length byte
    WORD max_response_time;					// Max time to wait for response (0-65535ms)
    BYTE min_buffer_space;					// Min receive buffer space allowed
    BYTE poll_msg[MAX_POLL_MSG_LENGTH];		// Poll message
    BYTE inhibit_msg[MAX_POLL_MSG_LENGTH];	// Device inhibit message
} RS232PollConfig;

#endif

