/*****************************************************************
*
*     Project:      Firefly X10 Board
*
*     File:         inpmuxdemo.cpp
*
*     Version:      1.0
*
*     Description:  Input multiplexer Demonstration
*
*     Notes:        Console application to demonstrate Input mutliplexing
*
*     Changes:
*     1.0 RJB       First version, 20/07/2004
*
*****************************************************************/


#include <stdio.h>
#ifndef X10_LINUX_BUILD
#include <conio.h>
#else
#include <ctype.h>
#include "linuxkb.c"
#endif
#include <time.h>
#include "fflyusb.h"
#include "unlockio.h"
#include "x10errhd.c"

#ifndef X10_LINUX_BUILD
#define InitialiseConsole()
#endif

static unsigned char qMuxEmabled = 0;

static void displayMultiplexedInputs( FireFlyUSB * FireFly );
static void togleMultiplexedInputs( FireFlyUSB * FireFly );


int main( int argc, char* argv[])
{
	FireFlyUSB FireFly;
	char key;

	printf( "Firefly X10 Board\n" );
	printf( "=================\n\n" );
	printf( "This application demonstrations X10 Input multiplexer.\n\n" );

	// Get a handle to the device
	printf( "Establishing link with FireFly USB device..." );
	if ( !FireFly.init( ) )
		return( ExitFunction( "Initialisation error.", &FireFly, 1 ) );
	UnlockX10( &FireFly );
	printf( "success.\n" );

    /* This is only used on the Linux version but #ifdefs look ugly in the
       code*/
	InitialiseConsole();

	while( TRUE )
	{
		printf( "X10 Multiplexed Inputs Menu\n" );
		printf( "===========\n\n" );

		printf( "Options:\n" );
        if( 0 == qMuxEmabled )
        {
            printf( "    'T' to Enable the input multiplexer.\n" );
        }
        else
        {
            printf( "    'T' to disable the input multiplexer.\n" );
        }
        
		printf( "    'P' to Display multiplexed inputs.\n" );
		printf( "or, 'C' to exit.\n\n" );

		printf( "Please make a selection: " );

		key = toupper( getch());
		printf( "\n\n" );
		if ( key == 'T' ) 
        {
            togleMultiplexedInputs( &FireFly );
        }
		else if ( key == 'P' )
        {
            displayMultiplexedInputs( &FireFly );
        }
		else if ( key == 'C' )
        {
            break;
        }
	}

	return( ExitFunction( "Leaving program.", &FireFly, 0 ));
}


static void displayMultiplexedInputs( FireFlyUSB * FireFly )
{
    usbMultiplexedInput MuxInputs;
    int key;
    int i;
    int c;
    
	while( TRUE )
	{
		if( !FireFly->GetMultiplexedInputs( &MuxInputs ))
		{
			printf( "\nFailure %d getting multiplexed inputs.\n", (int)FireFly->GetLastError( ) );
			break;
		}

		/* Print inputs to screen. */
        printf( "Multiplexed inputs %s\n", MuxInputs.byMuxStatus != 0 ? "enabled" : "disabled" );
		printf( "\nInputs   2    1    0" );
		for( i = 0; i < USB_MUXIN_CHANNELS; i++ )
        {
            printf( "\nchannel%d ", i + 1 );
            for( c = USB_MUXIN_BYTES; c > 0 ; c-- )
            {   
                printf( "%02X   ", MuxInputs.byMuxInp[ i ][ c - 1 ]);
            }
        } 

		printf( "\n\nPress any key to update display or 'C' to close.\n" );

		key = toupper( getch());
		if ( key == 'C' )
        {
            break;
        } 
	} 
}


static void togleMultiplexedInputs( FireFlyUSB *FireFly )
{

    if( 0 == qMuxEmabled )
    {        
        if( 0 == FireFly->InputMultiplexing( InputMultiplexEnabled ))
        {
            printf( "\nFailed in enabling input multiplexer\n" );
        }
        else
        {
            printf( "\nInput Multiplexer enabled\n" );
            qMuxEmabled = 1;
        }
    }
    else
    {
        if( 0 == FireFly->InputMultiplexing( InputMultiplexDisabled ))
        {
            printf( "\nFailed in disabling input multiplexer\n" );
        }
        else
        {
            printf( "\nInput Multiplexer disabled\n" );
            qMuxEmabled = 0;
        }
    }
}
