/*****************************************************************
*
*     Project:      Firefly X10 Board
*
*     File:         paralleldemo.cpp
*
*     Version:      1.1
*
*     Description:  Parallel IO Demonstration
*
*     Notes:        Console application to demonstrate parallel IO
*
*     Changes:
*     1.1 MJB       Removed upper pulse time check when counting
*                   input pulses. Now only the minimum pulse
*                   length time is checked, 19/02/2003
*     1.0 MJB       First version, 25/11/2002
*
*****************************************************************/



#include <stdio.h>
#ifndef X10_LINUX_BUILD
#include <conio.h>
#else
#include <ctype.h>
#include "linuxkb.c"
#endif
#include <time.h>
#include "fflyusb.h"
#include "unlockio.h"
#include "x10errhd.c"

void TestInputPulseReading( FireFlyUSB * FireFly );
void TestHopperCoinRelease( FireFlyUSB * FireFly );

int main(int argc, char* argv[])
{
	FireFlyUSB FireFly;
	char key;

	printf("If this appear I will believe\n");
	printf( "Firefly X10 Board\n" );
	printf( "=================\n\n" );
	printf( "This application demonstrations X10 Parallel IO.\n\n" );

	// Get a handle to the device
	printf( "Establishing link with FireFly USB device..." );
	if ( !FireFly.init( ) )
		return( ExitFunction( "error.", &FireFly, 1 ) );
	UnlockX10( &FireFly );
	printf( "success.\n" );

	#ifdef X10_LINUX_BUILD
	InitialiseConsole( );
	EnableGetchNewline( );
	#endif

	while ( TRUE )
	{
		printf( "X10 Parallel I/O Menu\n" );
		printf( "=====================\n\n" );

		printf( "Options:\n" );
		printf( "    'P' to test input pulse reading.\n" );
		printf( "    'H' to test parallel hopper coin release.\n" );
		printf( "or, 'C' to exit.\n\n" );

		printf( "Please make a selection: " );

		key = toupper( getch() );
		printf( "\n\n" );
		if ( key == 'P' ) TestInputPulseReading( &FireFly );
		else if ( key == 'H' ) TestHopperCoinRelease( &FireFly );
		else if ( key == 'C' ) break;
	}

	return( ExitFunction( "Leaving program.", &FireFly, 0 ) );
}

int GetNumber( char *displayString, int lowValue, int highValue )
{
	char tempString[10];
	BOOL valueOK = FALSE;
	int value;

	#ifdef X10_LINUX_BUILD
	EnableEcho( );
	#endif

	while( !valueOK )
	{
		printf( "%s (%d-%d):", displayString, lowValue, highValue );
		scanf( "%s", tempString );
		value = atoi( tempString );

		if ( ( value >= lowValue ) && ( value <= highValue ) ) valueOK = TRUE;
		else printf( "Incorrect value.\n\n" );
	}

	#ifdef X10_LINUX_BUILD
	DisableEcho( );
	#endif

	return( value );
}

void TestInputPulseReading( FireFlyUSB * FireFly )
{
	usbInputBitId inputNum;
	BYTE lowPulseTime, inputCounterValue;
	ActiveState activeState = High;
	BOOL valueOK = FALSE;
	char key;

	inputNum = (usbInputBitId)GetNumber( "Please enter input number to read", 0, 23 );
	lowPulseTime = (BYTE)GetNumber( "Please enter input pulse lower time in ms", 0, 254 );
	while( !valueOK )
	{
		printf( "Is input active High or Low (H/L): " );
		key = getch();
		if ( key == 'H' || key == 'h' ) { activeState = High; valueOK = TRUE; }
		else if ( key == 'L' || key == 'l' ) { activeState = Low; valueOK = TRUE; }
		else printf( "Incorrect response.\n\n" );
	}

	if ( !FireFly->ConfigurePulsedInput( inputNum, lowPulseTime, activeState ) )
	{
		printf( "ConfigurePulsedInput error - %d.\n\n", (int)FireFly->GetLastError( ) );
		return;
	}

	if ( !FireFly->BeginPulsedInputCheck( inputNum ) )
	{
		printf( "BeginPulsedInputCheck error - %d.\n\n", (int)FireFly->GetLastError( ) );
		return;
	}

	while ( TRUE )
	{
		printf( "\nPress: \n" );
		printf( "  'E' to reset input counter.\n" );
		printf( "  'D' to decrement input counter.\n" );
		printf( "  'R' to read input counter.\n" );
		printf( "  'C' to leave submenu.\n\n" );

		key = getch();
		if ( key == 'E' || key == 'e' )
		{
			printf( "Resetting input counter.\n" );
			if ( !FireFly->ResetPulsedInputCounter( inputNum ) )
			{
				printf( "ResetPulsedInputCounter error - %d.\n\n", (int)FireFly->GetLastError( ) );
				return;
			}
		}
		else if ( key == 'D' || key == 'd' )
		{
			printf( "Decrementing input counter.\n" );
			if ( !FireFly->DecrementPulsedInputCounter( inputNum ) )
			{
				printf( "DecrementPulsedInputCounter error - %d.\n\n", (int)FireFly->GetLastError( ) );
				return;
			}
		}
		else if ( key == 'R' || key == 'r' )
		{
			printf( "Reading input counter: " );
			if ( !FireFly->ReadPulsedInputCounter( inputNum, &inputCounterValue ) )
			{
				printf( "ReadPulsedInputCounter error - %d.\n\n", (int)FireFly->GetLastError( ) );
				return;
			}
			printf( "%d\n", inputCounterValue );
		}
		else if ( key == 'C' || key == 'c' ) break;
	}
}

void TestHopperCoinRelease( FireFlyUSB * FireFly )
{
	usbInputBitId inputNum;
	usbOutputBitId outputNum;
	BYTE lowPulseTime, coinsToRelease, coinsReleased;
	WORD coinTimeout;
	ActiveState activeState = High;
	BOOL valueOK = FALSE;
	ReleaseHopperCoinsStatus status;
	char key;

	inputNum = (usbInputBitId)GetNumber( "Please enter input number to read", 0, 23 );
	lowPulseTime = (BYTE)GetNumber( "Please enter input pulse lower time in ms", 0, 254 );
	while( !valueOK )
	{
		printf( "Is input active High or Low (H/L): " );
		key = getch();
		if ( key == 'H' || key == 'h' ) { activeState = High; valueOK = TRUE; }
		else if ( key == 'L' || key == 'l' ) { activeState = Low; valueOK = TRUE; }
		else printf( "Incorrect response.\n\n" );
	}
	printf( "\n" );
	outputNum = (usbOutputBitId)GetNumber( "Please enter output number", 0, 31 );
	coinTimeout = (WORD)GetNumber( "Coin Timeout in ms", 1, 8000 );
	coinsToRelease = (BYTE)GetNumber( "Number of coins to release", 0, 255 );

	printf( "Clearing previous input pulse counter...\n" );
	if ( !FireFly->ResetPulsedInputCounter( inputNum ) )
	{
		printf( "ResetPulsedInputCounter error - %d.\n\n", (int)FireFly->GetLastError( ) );
		return;
	}

	while ( TRUE )
	{
		printf( "Press: \n" );
		printf( "  'R' to release parallel hopper coins.\n" );
		printf( "  'S' to get parallel hopper status.\n" );
		printf( "  'C' to leave submenu.\n\n" );

		key = getch();
		if ( key == 'R' || key == 'r' )
		{
			printf( "Releasing parallel hopper coins...\n" );
			if ( !FireFly->ReleaseParallelHopperCoins( inputNum, lowPulseTime, activeState,
				                                       outputNum, coinTimeout, coinsToRelease ) )
			{
				printf( "ReleaseParallelHopperCoins error - %d.\n\n", (int)FireFly->GetLastError( ) );
				return;
			}
		}
		else if ( key == 'S' || key == 's' )
		{
			printf( "Getting parallel hopper status..." );
			if ( !FireFly->GetParallelHopperStatus( inputNum, &status, &coinsReleased ) )
			{
				printf( "GetParallelHopperStatus error - %d.\n\n", (int)FireFly->GetLastError( ) );
				return;
			}
			if ( status == Running ) printf( "Running, %d released.\n", coinsReleased );
			else if ( status == Success ) printf( "Success, %d released.\n", coinsReleased );
			else if ( status == Failure ) printf( "Failure, %d released.\n", coinsReleased );
		}
		if ( key == 'C' || key == 'c' ) break;
	}
}
