/*****************************************************************
*
*     Project:      Firefly X10 Board
*
*     File:         spidemo.cpp
*
*     Version:      1.0
*
*     Description:  SPI Demonstration
*
*     Notes:        Console application to demonstrate SPI standard
*
*     Changes:
*     1.0 MJB       First version, 2/5/2002
*
*****************************************************************/

#include <stdio.h>
#include <time.h>
#ifndef X10_LINUX_BUILD
#include <conio.h>
#else
#include "linuxkb.c"
#endif
#include "fflyusb.h"
#include "unlockio.h"
#include "x10errhd.c"

int main(int argc, char* argv[])
{
	FireFlyUSB FireFly;
	BYTE SPI_message[50], receive_message[50], counter, i, index;
	char key;

	printf( "Firefly X10 Board\n" );
	printf( "=================\n\n" );
	printf( "This program demonstrates the SPI standard.\n\n" );

	// Get a handle to the device
	printf( "Establishing link with FireFly USB device..." );
	if ( !FireFly.init( ) )
	{
		printf( "error - %d.\n\n", (int)FireFly.GetLastError( ) );
		return ( 1 );
	}
	UnlockX10( &FireFly );
	printf( "success.\n" );

	if ( !FireFly.EnableSPI( ) ) return( ExitFunction( "Failure enabling SPI.", &FireFly, 1 ) );

	#ifdef X10_LINUX_BUILD
	InitialiseConsole( );
	#endif

	counter = '0';
	i = 0;
	while ( TRUE )
	{
		sprintf( (char *)SPI_message, "HEBER %c", counter );

		printf( "Message to display on SEC device: '%s'\n", SPI_message );

		if ( !FireFly.SendSEC( 0x40, i, 7, SPI_message, 15, 0, receive_message ) )
			return( ExitFunction( "Failure sending SPI message.", &FireFly, 1 ) );

		printf( "SPI Received message = [ " );
		for ( index = 0; index < 4; index++ ) printf( "%02X ", receive_message[index] );
		printf( "]\n" );

		counter++;
		i++;
		if ( counter > '9' ) counter = '0';

		printf( "\n\nPress any key to update display or 'C' to close.\n" );

		key = getch();
		if ( key == 'C' || key == 'c' ) break;
	}

	return( ExitFunction( "Leaving program.", &FireFly, 0 ) );
}
