/*****************************************************************
*
*     Project:      Firefly X10 Board
*
*     File:         timedemo.cpp
*
*     Version:      1.2
*
*     Description:  Real-time Clock Demonstration
*
*     Notes:        Console application to demonstrate real time clock
*
*
*     Changes:
*     1.2 MJB       Added GetProductVersion(), 18/3/2003
*     1.1 MJB       Added serial number read and read and reset
*                   security switches, 19/4/2002
*     1.0 MJB       First version, 1/2/2002
*
*****************************************************************/

#include <stdio.h>
#ifndef X10_LINUX_BUILD
#include <conio.h>
#else
#include <ctype.h>
#include "linuxkb.c"
#endif
#include "fflyusb.h"
#include "unlockio.h"
#include "x10errhd.c"

int main( int argc, char* argv[] )
{
	FireFlyUSB FireFly;
	BYTE VersionProduct[20], VersionDll[20], Version8051[20], VersionPIC[20];
	BYTE SerialNumberPIC[20];
	BYTE open_switches, closed_switches;

	printf( "Firefly X10 Board\n" );
	printf( "=================\n\n" );
	printf( "This application demonstrates the real time clock on the security\n" );
	printf( "microcontroller..\n\n" );

	// Get a handle to the device
	printf( "Establishing link with FireFly USB device..." );
	if ( !FireFly.init( ) ) return( ExitFunction( "error.", &FireFly, 1 ) );
	UnlockX10( &FireFly );
	printf( "success.\n\n" );

	if ( !FireFly.GetProductVersion( VersionProduct ) )
		return( ExitFunction( "Failure obtaining product version.", &FireFly, 1 ) );

	if ( !FireFly.GetDllVersion( VersionDll ) )
		return( ExitFunction( "Failure obtaining DLL version.", &FireFly, 1 ) );

	if ( !FireFly.Get8051Version( Version8051 ) )
		return( ExitFunction( "Failure obtaining 8051 version.", &FireFly, 1 ) );

	if ( !FireFly.GetPICVersion( VersionPIC ) )
		return( ExitFunction( "Failure obtaining PIC version.", &FireFly, 1 ) );

	if ( !FireFly.GetPICSerialNumber( SerialNumberPIC ) )
		return( ExitFunction( "Failure obtaining PIC serial number.", &FireFly, 1 ) );

	printf( "Product version information:\n" );
	printf( "    Product version       : %s\n",   VersionProduct );
	printf( "    Windows driver version: %s\n",   VersionDll );
	printf( "    8051 software version : %s\n",   Version8051 );
	printf( "    PIC software version  : %s\n",   VersionPIC );
	printf( "    PIC serial number     : %s\n\n", SerialNumberPIC );


	if ( !FireFly.ReadAndResetSecuritySwitchFlags( &closed_switches, &open_switches ) )
		return( ExitFunction( "ReadAndResetSecuritySwitchFlags failed.", &FireFly, 1 ) );

	printf( "Switch flags: Closed = %02X, Open = %02X\n", closed_switches, open_switches );

	// Set the time only once
	DWORD time = 0x01020304;
	
	printf( "Setting time to 0x%04X%04X...\n", (int)((time >> 16) & 0x0000FFFF), (int)(time & 0x0000FFFF) );
	if ( !FireFly.SetClock( time ) )
		return( ExitFunction( "Set time failed.", &FireFly, 1 ) );

	printf( "\nPress any key to get the time or 'C' to close...\n" );

	#ifdef X10_LINUX_BUILD
	InitialiseConsole( );
	EnableGetchNewline( );
	#endif

	while ( TRUE )
	{
		if ( !FireFly.GetClock( &time ) )
			return( ExitFunction( "Get time failed.", &FireFly, 1 ) );

		printf( "\rTime: 0x%04X%04X", (int)((time >> 16) & 0x0000FFFF), (int)(time & 0x0000FFFF) );

		/* Wait for keystroke. */
		char key = getch( );
		if ( key == 'C' || key == 'c' ) break;
	}

	return( ExitFunction( "Leaving program.", &FireFly, 0 ) );
}
